@@ -1,22 +1,27 @@
-from PySide6.QtGui import QMouseEvent, Qt, QWindow
-from PySide6.QtWidgets import QListWidget, QSizePolicy
+from PySide6.QtCore import QSize
+from PySide6.QtGui import QMouseEvent, Qt
+from PySide6.QtWidgets import QListWidget, QScrollBar, QSizePolicy, QWidget
 
 from foundry.game.level.LevelRef import LevelRef
 from foundry.gui.ContextMenu import ContextMenu
 
 
 class ObjectList(QListWidget):
-    def __init__(self, parent: QWindow, level_ref: LevelRef, context_menu: ContextMenu):
+    def __init__(self, parent: QWidget, level_ref: LevelRef, context_menu: ContextMenu):
         super(ObjectList, self).__init__(parent=parent)
 
-        self.setSizePolicy(QSizePolicy.Maximum, QSizePolicy.Maximum)
+        self.setSizePolicy(QSizePolicy.MinimumExpanding, QSizePolicy.MinimumExpanding)
         self.setSizeAdjustPolicy(QListWidget.AdjustToContents)
+        self.setSelectionMode(QListWidget.ExtendedSelection)
+        scroll_bar = QScrollBar(self)
+        self.setVerticalScrollBar(scroll_bar)
+        self.setVerticalScrollBarPolicy(Qt.ScrollBarAsNeeded)
+        self.setWordWrap(True)
 
         self.setSelectionMode(self.ExtendedSelection)
 
         self.level_ref: LevelRef = level_ref
         self.level_ref.data_changed.connect(self.update_content)
-
         self.context_menu = context_menu
 
         self.itemSelectionChanged.connect(self.on_selection_changed)
@@ -32,6 +37,9 @@ class ObjectList(QListWidget):
             "listed in a separate list below."
         )
 
+    def sizeHint(self):
+        return QSize(100, 200)
+
     def mousePressEvent(self, event: QMouseEvent):
         if event.button() == Qt.RightButton:
             self.on_right_down(event)
@@ -70,6 +78,13 @@ class ObjectList(QListWidget):
         self.context_menu.as_list_menu().popup(event.globalPos())
 
     def update_content(self):
+        """
+        A new item has been selected, so select the new item.
+        """
+        prior_selection = self.currentIndex()
+        currently_selected = set(obj.row() for obj in self.selectedIndexes())
+        ignore_prior_selection = False
+
         level_objects = self.level_ref.level.get_all_objects()
 
         labels = [obj.name for obj in level_objects]
@@ -82,15 +97,19 @@ class ObjectList(QListWidget):
 
         for index, level_object in enumerate(level_objects):
             item = self.item(index)
-
             item.setData(Qt.UserRole, level_object)
             item.setSelected(level_object.selected)
+            if level_object.selected and index not in currently_selected:
+                ignore_prior_selection = True
 
         self.blockSignals(False)
 
         if self.selectedIndexes():
             self.scrollTo(self.selectedIndexes()[-1])
 
+        if not ignore_prior_selection:
+            self.setCurrentIndex(prior_selection)
+
     def selected_objects(self):
         return [self.item(index.row()).data(Qt.UserRole) for index in self.selectedIndexes()]
 
